@@ -33,6 +33,7 @@
 #include "Gfitter/GTheoryFactory.h"
 #include "Gfitter/GFitterBase.h"
 #include "Gfitter/GMinuitFitter.h"
+#include "Gfitter/GMultiNestFitter.h"
 #include "Gfitter/GGeneticFitter.h"
 #include "Gfitter/GSimulatedAnnealing.h"
 #include "Gfitter/GGSLMinimizer.h"
@@ -135,6 +136,9 @@ void Gfitter::GController::Initialise()
       TString fitName(objstring->GetString()); fitName.ToLower();
       if (fitName == "minuit") { 
         fitter = new GMinuitFitter();
+      }
+      else if (fitName == "multinest") {
+	fitter = new GMultiNestFitter();    
       } else if (fitName == "gslminimizer") {
          if (fitArr->GetEntries()==2) {
            TString myType = ((TObjString*)fitArr->At(1))->GetString();
@@ -169,6 +173,7 @@ void Gfitter::GController::Initialise()
       if (myPreFitterName == "genetic") preFitter = new GGeneticFitter();
       else if (myPreFitterName == "annealing") preFitter = new GSimulatedAnnealing();
       else if (myPreFitterName == "randomsimplex") preFitter = new GRandomSimplex();
+      else if (myPreFitterName == "multinest") preFitter = new GMultiNestFitter();
       else { // nothing else implemented at present
          m_logger << kFATAL <<  "<Initialise> Unknown prefitter: \"" << preFitterName << "\"" << GEndl;
       }
@@ -219,16 +224,18 @@ void Gfitter::GController::ExecuteAllActions()
       lowerActionName.ToLower();
             
       // perform actions
-      if      (lowerActionName == "evaluate"      ) EvaluateAction();
-      else if (lowerActionName == "evaluatetoy"   ) EvaluateToyAction();
-      else if (lowerActionName == "contour2d"     ) Contour2DAction();
-      else if (lowerActionName == "fctoffreepara" ) FctOfFreeParaAction();
-      else if (lowerActionName == "scan1d"        ) Scan1DAction();
-      else if (lowerActionName == "scananalysis"  ) ScanAnalysisAction();
-      else if (lowerActionName == "scan2d"        ) Scan2DAction();
-      else if (lowerActionName == "toyscan1d"     ) ToyScan1DAction();
-      else if (lowerActionName == "toyscan2d"     ) ToyScan2DAction();
-      else if (lowerActionName == "validate"      ) ValidateAction();
+      if      (lowerActionName == "evaluate"           ) EvaluateAction();
+      else if (lowerActionName == "evaluatetoy"        ) EvaluateToyAction();
+      else if (lowerActionName == "contour2d"          ) Contour2DAction();
+      else if (lowerActionName == "fctoffreepara"      ) FctOfFreeParaAction();
+      else if (lowerActionName == "scan1d"             ) Scan1DAction();
+      else if (lowerActionName == "propagateerror1d"   ) PropagateError1DAction();
+      else if (lowerActionName == "scananalysis"       ) ScanAnalysisAction();
+      else if (lowerActionName == "scan2d"             ) Scan2DAction();
+      else if (lowerActionName == "scannd"             ) ScanNDAction();
+      else if (lowerActionName == "toyscan1d"          ) ToyScan1DAction();
+      else if (lowerActionName == "toyscan2d"          ) ToyScan2DAction();
+      else if (lowerActionName == "validate"           ) ValidateAction();
       else {
          m_logger << kFATAL << "Unknown action: \"" << lowerActionName << "\"" << GEndl;
       }
@@ -267,6 +274,12 @@ void Gfitter::GController::EvaluateAction()
    Double_t chi2 = gStore()->GetFitter()->ExecuteFit();
    gStore()->GetFitter()->UpdateResults();
 
+   m_logger << kINFO << "Printint after-fit information: " << GEndl;
+
+   // print parameters and chi2 after fit
+   gStore()->DumpParameters();
+   gStore()->GetFitter()->DumpParameters();
+
    // store fit results in a tree
    TTree* FitResults = new TTree( "FitResults", "FitResults" );
     
@@ -418,6 +431,24 @@ void Gfitter::GController::Scan1DAction()
    delete scanner;
 }
 
+void Gfitter::GController::PropagateError1DAction()
+{
+  // first do fit
+  Gfitter::GController::EvaluateAction();
+
+  // scan parameter
+  GScan* scanner = new GScan();
+  
+  // loop over parameters for this action
+  std::vector<TString>::const_iterator it = m_pars.begin();
+  for (it = m_args.begin(); it != m_args.end(); it++) {
+    m_logger << kINFO << "The 1D-propagation variable is: \"" << (*it) << "\"" << GEndl;
+    scanner->PropagateError1D( (*it) );
+  }
+  
+  delete scanner;
+}
+
 void Gfitter::GController::Scan2DAction()
 {
    // scan parameter
@@ -458,6 +489,87 @@ void Gfitter::GController::Scan2DAction()
    delete scanner;
 }
 
+void Gfitter::GController::ScanNDAction(){
+
+  GMultiNestFitter* fitter = dynamic_cast<GMultiNestFitter*>(gStore()->GetFitter());
+  if(!fitter){
+    m_logger << kFATAL << "ScanNDAction is only compatible with multinest fitter. Use FitterType = \"T:multinest\" in your configuration!" << GEndl;
+    return;
+  }
+
+     // global fit with multinest
+   gStore()->GetFitter()->Initialise();
+   gStore()->GetFitter()->ExecuteFit();
+   gStore()->GetFitter()->UpdateResults();
+
+   // convert the 2D Fortran arrays to C++ arrays
+
+   // the posterior distribution
+   // postdist will have nPar parameters in the first nPar columns & loglike value & the posterior probability in the last two columns
+   
+   int i, j;
+   int nPar = fitter->GetNPar();
+
+   if( (int)gStore()->GetActiveParameters().size() != nPar){
+     m_logger << kFATAL << "Something went wrong: number of parameters used by multinest differs from number of parameters given in gStore." << GEndl;
+     return;
+   }
+
+   int nSamples = fitter->GetNSamples();
+   int nlive = fitter->GetNLive();
+   std::vector<double> posterior = fitter->GetPosterior();
+   std::vector<double> physLive = fitter->GetLivePoints();
+   double postdist[nPar + 2];
+
+   TTree* ScanResults = new TTree( "ScanResults", "ScanResults" );
+
+   // create the branches 
+   i=0;
+   GParPtrVec_t::const_iterator par = gStore()->GetActiveParameters().begin();
+   for (par = gStore()->GetActiveParameters().begin(); par != gStore()->GetActiveParameters().end(); par++) {
+     ScanResults->Branch( (*par)->GetFullNameWONSpace(), &postdist[i], "ParVal/D" );
+     ++i;
+   }
+   ScanResults->Branch( "loglike", &postdist[nPar], "LLVal/D" );
+   ScanResults->Branch( "posterior", &postdist[nPar+1], "PostVal/D" );
+
+   for( j = 0; j < nSamples; j++ ){
+     for( i = 0; i < nPar + 2; i++ ){
+       postdist[i] = posterior[i * nSamples + j];
+     }
+     ScanResults->Fill();
+   }
+
+   // last set of live points
+   // pLivePts will have nPar parameters in the first nPar columns & loglike value in the last column
+   
+   double pLivePts[nPar + 1];
+
+   TTree* LivePointsResults = new TTree( "LivePoints", "LivePoints" );
+
+   i=0;
+   par = gStore()->GetActiveParameters().begin();
+   for (par = gStore()->GetActiveParameters().begin(); par != gStore()->GetActiveParameters().end(); par++) {
+     LivePointsResults->Branch( (*par)->GetFullNameWONSpace(), &pLivePts[i], "ParVal/D" );
+     ++i;
+   }
+   LivePointsResults->Branch( "loglike", &pLivePts[nPar], "LLVal/D" );
+
+   for( j = 0; j < nlive; j++ ){
+     for( i = 0; i < nPar+1; i++ ){
+       pLivePts[i] = physLive[i * nlive + j];
+     }
+     LivePointsResults->Fill();
+   }
+   
+
+   // m_logger << kINFO << "Elapsed time: " << fitter->GetFitDuration() << GEndl;    
+   
+   ScanResults->Write();
+   LivePointsResults->Write();
+   delete ScanResults;
+}
+
 void Gfitter::GController::ScanAnalysisAction()
 {
    // scan parameter
