@@ -16,6 +16,7 @@
 #include "TXMLNode.h"
 #include "TXMLDocument.h"
 #include "TXMLAttr.h"
+#include <algorithm> 
 
 #include "Gfitter/GDCardInterpreterXML.h"
 #include "Gfitter/GUtils.h"
@@ -35,7 +36,7 @@ using namespace std;
 // reference list for allowed attributes of XML nodes
 const TString GDCardInterpreterXML_Separator( ":" );
 // CanScanned only foe backward compatibility
-const TString GDCardInterpreterXML_Attributes_Parameter( "Name:Alias:ModifyInToy:Value:CanScanned:Active:File:Reference:ChiSq:FitLimits:PreFitScan:ScanRange:Rescale:RefValue:WorkspaceFile:WorkspaceName:ModelConfig" );
+const TString GDCardInterpreterXML_Attributes_Parameter( "Name:Alias:ModifyInToy:Value:CanScanned:Active:File:Reference:ChiSq:Axes:ForceChi2OfZero:FitLimits:PreFitScan:ScanRange:Rescale:RefValue:WorkspaceFile:WorkspaceName:ModelConfig" );
 
 Gfitter::GDCardInterpreterXML::GDCardInterpreterXML( const TString& xmlFileName )
    : GDCardInterpreterBase( xmlFileName )
@@ -52,6 +53,7 @@ Bool_t Gfitter::GDCardInterpreterXML::Interpret()
    m_logger << kINFO << "Read xml data card: \"" << m_dataCardName << "\"" << GEndl;
   
    TDOMParser* xmlparser = new TDOMParser();
+   xmlparser->SetReplaceEntities( kTRUE );
    xmlparser->SetValidate(kFALSE); // dont check dtd file consistency
       
    Int_t parseCode = xmlparser->ParseFile( m_dataCardName );
@@ -121,6 +123,9 @@ void Gfitter::GDCardInterpreterXML::ReadData( TXMLNode* dataNode )
 
    // retrieve childrens of node
    TXMLNode* node = dataNode->GetChildren();
+
+   std::vector<TString> parameter_names;
+   std::vector<TString> parameter_aliases;
    for (; node != 0; node = node->GetNextNode()) {
       if (TString("Parameter") != node->GetNodeName()) continue;
 
@@ -154,6 +159,18 @@ void Gfitter::GDCardInterpreterXML::ReadData( TXMLNode* dataNode )
       TString alias = name;
       curAttr = (TXMLAttr*)attribs->FindObject( "Alias" );         
       if (curAttr != 0) alias = curAttr->GetValue();
+
+
+      //check if parameter has already been declared
+      std::vector<TString>::iterator it = find( parameter_names.begin(), parameter_names.end(), name);
+      std::vector<TString>::iterator it2 = find( parameter_aliases.begin(), parameter_aliases.end(), alias);
+      if (it != parameter_names.end() && it2 != parameter_aliases.end()){
+	m_logger << kINFO << "Parameter " << name << " already in input. Skip this entry." << GEndl;
+	continue;
+      }
+      parameter_names.push_back(name);
+      parameter_aliases.push_back(alias);
+
       
       // should the parameter modify during Toy analyis by value*sigma
       TString modifyintoy = "0";
@@ -339,7 +356,31 @@ void Gfitter::GDCardInterpreterXML::ReadData( TXMLNode* dataNode )
          if (curAttr == 0) m_logger << kFATAL << "No \"ChiSq\" attribute given" << GEndl;
          TString formula   = curAttr->GetValue();
 
-         dataCurve = new GDataCurve(fileName, histName, formula);	 
+         curAttr = (TXMLAttr*)attribs->FindObject( "Axes" );
+         if (curAttr == 0) {
+             m_logger << kINFO << "Assuming 1D distribution" << GEndl;
+             dataCurve = new GDataCurve(fileName, histName, formula);	 
+         } else {
+             m_logger << kINFO << "Assuming 2D distribution" << GEndl;
+             TString axisNames = curAttr->GetValue();
+             // check if there is axis definition, if not pickup right axis according to name
+             GStringList axes_list(axisNames);
+             if (axes_list[0] != "x" || axes_list[0] != "X" || axes_list[0] != "y" || axes_list[0] != "Y" ){
+                 if      (axes_list[0] == name) axisNames = "x:"+axisNames;
+                 else if (axes_list[1] == name) axisNames = "y:"+axisNames;
+                 else m_logger<<kFATAL<<"Wrong format in Axes definition: No axis name corresponds to parameter \""  << name << "\"" << GEndl;
+             }
+             m_logger << kINFO << "axis names: \"" << axisNames << "\"" << GEndl;
+
+             /// Check if this needs to be a dummy chi2.
+	     curAttr = (TXMLAttr*)attribs->FindObject( "ForceChi2OfZero" );
+             Bool_t forceChi2OfZero(false);
+             if (curAttr != 0) {
+                 forceChi2OfZero = (curAttr->GetValue() == TString("T") ? kTRUE : kFALSE);
+             }
+             dataCurve = new GDataCurve(fileName, histName, formula, axisNames, forceChi2OfZero);	 
+         }
+
       } 
       else if ((curAttr = (TXMLAttr*)attribs->FindObject( "WorkspaceFile" )) != 0) {
 	
@@ -364,12 +405,12 @@ void Gfitter::GDCardInterpreterXML::ReadData( TXMLNode* dataNode )
       TXMLAttr*  srAtt     = (TXMLAttr*)attribs->FindObject("ScanRange");
       GInterval* scanRange = 0;
       if (srAtt != 0) scanRange = new GInterval( srAtt->GetValue() );
-
+		
       // create the parameter
       GParameter* par=0;
-      if      (data)      par = new GParameter( name, alias, active, data,      scanRange );
-      else if (dataCurve) par = new GParameter( name, alias, active, dataCurve, scanRange );
-      else if (dataWorkspace) par = new GParameter( name, alias, active, dataWorkspace, 0 );
+      if      (data)          par = new GParameter( name, alias, active, data         , scanRange );
+      else if (dataCurve)     par = new GParameter( name, alias, active, dataCurve    , scanRange );
+      else if (dataWorkspace) par = new GParameter( name, alias, active, dataWorkspace, 0         );
       else {
          m_logger << kFATAL << "Cannot create parameter \"" << name 
                   << "\" neither Data nor DataCurve nor Workspace available!" << GEndl;
@@ -383,8 +424,13 @@ void Gfitter::GDCardInterpreterXML::ReadData( TXMLNode* dataNode )
       }         
 
       // are fit limits given ?
-      curAttr = (TXMLAttr*)attribs->FindObject( "FitLimits" );         
-      if (curAttr != 0) par->SetFitRange( new GInterval( curAttr->GetValue() ) );
+      curAttr = (TXMLAttr*)attribs->FindObject( "FitLimits" );    
+      GInterval* fitRange = 0;
+      if (curAttr != 0){ 
+			fitRange = new GInterval( curAttr->GetValue() );
+			m_logger << kINFO << "Overwriting fit range of parameter " << name << " to [" << fitRange->GetMin() << "," << fitRange->GetMax() << "]" << GEndl;
+			par->SetFitRange( fitRange );
+		}
 
       // is a pre-fit scan range given ?
       curAttr = (TXMLAttr*)attribs->FindObject( "PreFitScan" );         
