@@ -113,10 +113,10 @@ void Gfitter::GMinuitFitter::Initialise()
          if ((*par)->GetFitStep() <= 0) m_logger << kFATAL << "Huge troubles with fit step of par: " 
                                                  << (*par)->GetParName() << GEndl;
     
-         //m_logger << kINFO << (*par)->GetParName() 
-         //         << ", MinFitRange : " << (*par)->GetFitRange().GetMin()
-         //         << ", MaxFitRange : " << (*par)->GetFitRange().GetMax()
-         //         << GEndl;
+         m_logger << kINFO << (*par)->GetParName() 
+                  << ", MinFitRange : " << (*par)->GetFitRange().GetMin()
+                  << ", MaxFitRange : " << (*par)->GetFitRange().GetMax()
+                  << GEndl;
                   
          // book the fit parameter
          SetParameter( npars++, 
@@ -241,7 +241,8 @@ Double_t Gfitter::GMinuitFitter::ExecuteFit()
       }
    }
 
-   if (false) {
+   if (true) {
+//   if (false) {
       m_logger << kDEBUG << "=====================================" << GEndl;
       GParPtrVec_t::const_iterator par;
       const GParPtrVec_t& v = gStore()->GetActiveParameters();
