@@ -39,6 +39,95 @@ Gfitter::GScan::GScan()
 Gfitter::GScan::~GScan()
 {}
 
+void Gfitter::GScan::PropagateError1D( const TString& par )
+{
+   GParameter* gpar = gStore()->GetParameter( par );
+   if (gpar == 0) {
+      m_logger << kFATAL 
+               << "<PropagateErrors1D> Unknown scan parameter \"" << par << "\": " << " check datacard" << GEndl;
+   }     
+
+   PropagateError1D( gpar );
+}
+
+void Gfitter::GScan::PropagateError1D( GParameter* gpar )
+{
+   // sanity check
+   if (gpar == 0) m_logger << kFATAL << "<PropagateErrors1D> Zero GParameter pointer" << GEndl;
+
+   m_logger << kINFO << "Scanning parameter \"" << gpar->GetParName() << "\""
+            << " (full name: \"" << gpar->GetFullName() << "\")"
+            << " in interval [" << gpar->GetScanRange().GetMin() << ", " << gpar->GetScanRange().GetMax() << "]"
+            << GEndl;
+
+
+   if (true) {
+
+     double scanval; // = gpar->GetFitValue();
+
+     if (!gpar->HasTheory()) scanval = (gpar)->GetFitValue();
+     else                    scanval = (gpar)->GetTheoryPrediction();
+
+     GParPtrVec_t::const_iterator par;
+     const GParPtrVec_t& v = gStore()->GetActiveParameters();
+     
+     // debug output
+     for (par = v.begin(); par != v.end(); par++) {
+       Double_t value;
+       if (!(*par)->HasTheory()) {
+	 value = (*par)->GetFitValue();
+	 double errp = (*par)->GetErrGaussp();
+	 double errm = (*par)->GetErrGaussm();
+
+	 /*
+                     << (*par)->GetResult()->GetValue()  << " | " << noshowpos 
+                     << (*par)->GetResult()->GetErrGaussSym(1) << " | " << showpos 
+                     << (*par)->GetResult()->GetErrGaussAsym(1) << " | " << showpos 
+                     << (*par)->GetResult()->GetErrGaussAsym(-1) << " | " << noshowpos 
+	 */
+	 if (errp==0) {
+	   if ( (*par)->GetResult()!=0 ) errp = (*par)->GetResult()->GetErrGaussSym(1);
+	 }
+	 if (errm==0) {
+	   if ( (*par)->GetResult()!=0 ) errm = (*par)->GetResult()->GetErrGaussSym(1);
+	 }
+
+	 (*par)->SetFitValue( value + errp );
+	 double scanerrp; // = gpar->GetFitValue() - scanval;
+	 if (!gpar->HasTheory()) scanerrp = (gpar)->GetFitValue() - scanval;
+	 else                    scanerrp = (gpar)->GetTheoryPrediction() - scanval;
+
+	 (*par)->SetFitValue( value - errm );
+	 double scanerrm; // = gpar->GetFitValue() - scanval;
+	 if (!gpar->HasTheory()) scanerrm = (gpar)->GetFitValue() - scanval;
+	 else                    scanerrm = (gpar)->GetTheoryPrediction() - scanval;
+
+	 // and reset
+	 (*par)->SetFitValue( value );
+
+	 m_logger << kINFO << "Fit par " << (*par)->GetFullName()
+		  << "\" initial value (v/fit): ("
+		  << (*par)->GetValue() << "/" << value << ")"
+		  << ", error range: [" << -errm << ", "
+		  << errp << "]"
+		  << ", impact on " << gpar->GetFullName() << " : [" << scanerrm << ", "
+		  << scanerrp << "]"
+		  << ", chi2: " << (*par)->GetChiSquared()
+		  << GEndl;
+	 if (TMath::IsNaN( value )) m_logger << kFATAL << "NAN !!!" << GEndl;
+       }
+     }
+   }
+      
+   m_logger << kINFO << "Error propagation is complete!" << GEndl;
+}
+
+
+
+
+
+
+
 void Gfitter::GScan::Scan1D( const TString& par, Int_t nbins )
 {
    GParameter* gpar = gStore()->GetParameter( par );
@@ -62,7 +151,6 @@ void Gfitter::GScan::Scan1D( GParameter* gpar )
             << GEndl;
    m_logger << kINFO << "Number of steps in scan: " << gpar->GetNbins() << GEndl;
 
-      
    // book parameter tree
    TTree* FitTree = new TTree( Form( "results_%s", gpar->GetParNameWONSpace().Data() ), 
                                Form( "Results for parameter %s", gpar->GetParNameWONSpace().Data() ) );
@@ -89,6 +177,8 @@ void Gfitter::GScan::Scan1D( GParameter* gpar )
    Double_t x_StartErrm[num];
    Int_t i = 0;
 
+   Double_t scan_val;
+
    for (GParPtrVec_t::const_iterator par1 = gStore()->GetActiveParameters().begin(); 
         par1 != gStore()->GetActiveParameters().end(); par1++) {
       FitTree  ->Branch( (*par1)->GetFullNameWONSpace() , &x_val[i], "x_val/D" );
@@ -103,6 +193,9 @@ void Gfitter::GScan::Scan1D( GParameter* gpar )
    FitTree->Branch("chi2_tot" , &x_chi2tot, "x_chi2tot/D" );
    Forward->Branch("chi2_tot" , &x_chi2tot, "x_chi2tot/D" );
 
+   FitTree->Branch( Form( "scan_%s", gpar->GetParNameWONSpace().Data() ) , &scan_val, "scan_val/D" );
+   Forward->Branch( Form( "scan_%s", gpar->GetParNameWONSpace().Data() ) , &scan_val, "scan_val/D" );
+
    // backup paramter settings
    gpar->Backup();
   
@@ -142,6 +235,7 @@ void Gfitter::GScan::Scan1D( GParameter* gpar )
       for (Int_t ibin = ibinStart; ibin != ibinEnd; ibin += istep) {
         
          gpar->SetFitValue( th1Chi2->GetBinCenter( ibin ) );
+         scan_val = th1Chi2->GetBinCenter( ibin );
          
          // perform the fit
          Double_t chi2  = gStore()->GetFitter()->ExecuteFit();
@@ -254,6 +348,7 @@ void Gfitter::GScan::Scan1D( GParameter* gpar )
    delete StartTree;
    
    gpar->SetResult( m_result );	
+
    m_logger << kINFO << "Scan is completed!" << GEndl;
 }
 
