@@ -436,3 +436,16 @@ TString Gfitter::GUtils::ReplaceRegularExpressions( const TString& s, const TStr
 
    return snew;
 }
+
+
+double 
+Gfitter::ground(double x, unsigned int digits) {
+  if (digits > 0) {
+    return ground(x*10.0, digits-1)/10.0;
+  }
+  else {
+    return round(x);
+  }
+}
+
+
