@@ -28,5 +28,7 @@
 #pragma link C++ class Gfitter::GRandomSimplex;
 #pragma link C++ class Gfitter::GCornerSimplex;
 #pragma link C++ class Gfitter::GWorkspace;
+#pragma link C++ class Gfitter::GMultiNestFitter;
 
+#pragma link C++ function Gfitter::ground;
 #endif
